@@ -1,5 +1,5 @@
 /*
-x86 not-threaded, PL_op in ebx, PL_sig_pending temp in 4(%ebp)
+x86 not-threaded, PL_op in eax, PL_sig_pending temp in ecx
 
 prolog:
 	55                   	push   %ebp
@@ -13,13 +13,9 @@ dispatch_getsig:
 	8b 0d xx xx xx xx	mov    $PL_sig_pending,%ecx
 dispatch:
 	85 c9                	test   %ecx,%ecx
-	74 06                	je     +5
+	74 06                	je     +6
 	e8 xx xx xx xx          call   *Perl_despatch_signals #relative
 
-maybranch:
-	mov 4(%ebx),%edx	; save op->next
-
-
 
 
 epilog:
@@ -32,31 +28,36 @@ epilog:
 /* Usage: sizeof(PROLOG) + PUSHc(PROLOG) */
 
 T_CHARARR x86_prolog[] = {
-    enter_8,
-    push_ebx,
-    push_edx,
-    mov_mem_ebx(0)	/* &PL_op  */
+    push_ebp,		/* save frame pointer*/
+    mov_esp_ebp,	/* set new frame pointer */
+    push_ebx,		/* &PL_op  */
+    push_ecx,		/* reserve */
+    sub_x_esp(8),	/* room for 2 locals: &PL_sig_pending and op */
+    mov_mem_rebx(0)    /* &PL_op to ebx */
 #ifdef HAVE_DISPATCH
-    ,mov_mem_4ebp(0)	/* &PL_sig_pending */
+    ,mov_mem_4ebp(0)     /* &PL_sig_pending to -4(%ebp) */
 #endif
 };
 
 unsigned char * push_prolog(unsigned char *code) {
     unsigned char prolog[] = {
-        enter_8,
-        push_ebx,	/* &PL_op */
-        push_edx,	/* needed temp */
-        mov_mem_ebx(&PL_op) /* %ebx *IS* preserved across function calls */
+        push_ebp,
+        mov_esp_ebp,
+        push_ebx,
+        push_ecx,
+        sub_x_esp(8),
+        mov_mem_rebx(&PL_op)
 #ifdef HAVE_DISPATCH
-        ,mov_mem_4ebp(&PL_sig_pending)
+	,mov_mem_4ebp(&PL_sig_pending)
 #endif
-    };
+ };
     PUSHc(prolog);
     return code;
 }
 
 T_CHARARR x86_epilog[] = {
-    pop_edx,
+    add_x_esp(8),
+    pop_ecx,
     pop_ebx,
     leave,		/* restore esp */
     ret
@@ -65,59 +66,41 @@ T_CHARARR x86_epilog[] = {
 T_CHARARR x86_call[]  = {0xe8};      	/* call near offset $PL_op->op_ppaddr */
 T_CHARARR x86_jmp[]   = {0xff,0x25}; 	/* jmp *$PL_op->op_ppaddr */
 T_CHARARR x86_save_plop[]  = {
-    mov_eax_rebx			/* &PL_op in %ebx */
+    mov_eax_8ebp			/* &PL_op in -8(%ebp) */
 };
+T_CHARARR x86_dispatch_getsig[] = {
+    0x8b,0x0d		/* mov $PL_sig_pending,%ecx */
+}; /* &PL_sig_pending abs */
 T_CHARARR x86_dispatch[] = {
-    mov_4ebp_edx,	/* value of PL_sig_pending from 4(%ebp) (ptr) to %eax */
-    mov_redx_eax,
-    test_eax_eax,
-    je(5)  		/* je     +5 */
-};      		/* call   Perl_despatch_signals */
+    0x85,0xc9,	/* test   %ecx,%ecx */
+    0x74,0x06,  /* je     +6 */
+    0xe8};      /* call   Perl_despatch_signals */
+/* &Perl_despatch_signals relative */
+T_CHARARR x86_dispatch_post[] = {}; /* fails with msvc */
 
 T_CHARARR maybranch_plop[] = {
-    mov_mem_rebp8,fourbyte
+    mov_mem_rebx(0),
+    mov_eax_8ebp
 };
-CODE *
-push_maybranch_plop(CODE *code, OP* next) {
-    CODE maybranch_plop[] = {
-	mov_mem_rebp8};
+unsigned char *
+push_maybranch_plop(unsigned char *code) {
+    unsigned char maybranch_plop[] = {
+	mov_mem_rebx(&PL_op),
+	mov_eax_8ebp};
     PUSHc(maybranch_plop);
-    PUSHrel(&next);
     return code;
 }
-T_CHARARR maybranch_check[] = {
-    cmp_eax_rebp8,
-    je_0, fourbyte
-};
-T_CHARARR maybranch_checkw[] = {
-    cmp_eax_rebp8,
-    jew_0, fourbyte
-};
 
-CODE *
-push_maybranch_check(CODE *code, int next) {
-    CODE maybranch_check[] = {
-	cmp_eax_rebp8,
-	je_0};
-    if (abs(next) > 128) {
-        CODE maybranch_checkw[] = {
-            cmp_eax_rebp8,
-            jew_0};
-        PUSHc(maybranch_checkw);
-        PUSHrel((CODE*)next);
-    } else {
-        PUSHc(maybranch_check);
-        PUSHbyte(next);
-    }
-    return code;
-}
 
 # define PROLOG 	x86_prolog
 # define EPILOG         x86_epilog
 # define CALL	 	x86_call
 # define JMP	 	x86_jmp
 # define SAVE_PLOP	x86_save_plop
+# define DISPATCH_GETSIG x86_dispatch_getsig
 # define DISPATCH       x86_dispatch
+# define DISPATCH_POST  x86_dispatch_post
+
 
 /*
  * Local variables:
