@@ -39,88 +39,68 @@ epilog after final Perl_despatch_signals
 	c3                   	ret
 */
 
-/* my_perl already on stack, Iop at 4(%ebx) */
-T_CHARARR x86thr_prolog[] = { 
-    enter_8,
-    push_ebx,		/* &my_perl */
-    push_edx,		/* needed temp */
-    mov_rebp_ebx(8)     /* mov 0x8(%ebp),%ebx my_perl */
-#ifdef HAVE_DISPATCH
-    ,mov_mem_4ebp(0)
-#endif
-};
+/* my_perl already on stack, Iop at 4(%ebx),  */
+#define X86THR_PROLOG					\
+    push_ebp,		/* save frame pointer*/		\
+    mov_esp_ebp,	/* set new frame pointer */ 	\
+    push_edi,						\
+    push_esi,						\
+    push_ebx,		/* &my_perl */			\
+    push_ecx,						\
+    sub_x_esp(8),	/* room for 2 locals: op, p */ 	\
+    mov_eax_ebx
+static unsigned x86thr_prolog[] = { X86THR_PROLOG };
 unsigned char *push_prolog(unsigned char *code) {
-    unsigned char prolog[] = {
-        enter_8,
-        push_ebx,	   /* &my_perl */
-        push_edx,	   /* needed temp */
-        mov_rebp_ebx(8)    /* mov 0x8(%ebp),%ebx my_perl */
-#ifdef HAVE_DISPATCH
-        ,mov_mem_4ebp(&PL_sig_pending)
-#endif
-    };
-    PUSHc(prolog);
+    PUSHc(x86thr_prolog);
     return code;
 }
 
 T_CHARARR x86thr_epilog[] = {
-    pop_edx,
+    add_x_esp(8),
+    pop_ecx,
     pop_ebx,
+    pop_esi,
+    pop_edi,
     leave,		/* restore esp, ebp */
     ret
 };
 
-/* call near with my_perl (ptr at %ebx) as arg1.
-   push my_perl, call near offset $PL_op->op_ppaddr 
- */
+/* call near */
 T_CHARARR x86thr_call[]  = {
     0x89,0x1c,0x24,	/* mov    %ebx,(%esp) */
-    0xE8		/* call near offset */
+    0xE8		/* call near 0xoffset */
 };
-/* after each call: PL_op = eax. PL_op is at my_perl->Iop, so update it for threading concurrency. */
+/* push my_perl, call near offset $PL_op->op_ppaddr */
 T_CHARARR x86thr_save_plop[] = {
     0x89,0x43,0x04	/* mov    %eax,0x4(%ebx) */
 }; /* save new PL_op into my_perl */
 T_CHARARR x86_nop[]          = {0x90};         /* pad */
+/* T_CHARARR x86thr_dispatch_getsig[] = {}; */ /* empty decl fails with msvc */
 T_CHARARR x86thr_dispatch[] = {
-    mov_4ebp_edx,	/* value of PL_sig_pending from 4(%ebp) (ptr) to %eax */
-    mov_redx_eax,
-    test_eax_eax,
-    je(8)  		/* je     +8 */
-}; /* call   Perl_despatch_signals */
+    0x89,0x1e,0x89,0x46,
+    0x04,0x8b,0x86,0x84,
+    0x03,0x00,0x00,0x85,
+    0xC0,0x74,0x08,0x89,
+    0x34,0x24,0xFF,0x25
+}; /* check and call $Perl_despatch_signals */
+/* after calling Perl_despatch_signals, restore my_perl into ebx and push for next.
+   restore my_perl into ebx and push */
+T_CHARARR x86thr_dispatch_post[] = {
+    0x83,0xc4,0x10,0x83,
+    0xec,0x0c,0x31,0xdb,
+    0x53,0x90
+};
 
 T_CHARARR maybranch_plop[] = {
-    mov_mem_rebp8,fourbyte
+    mov_mem_rebx(0),
+    mov_eax_8ebp
 };
-CODE *
-push_maybranch_plop(CODE *code, OP* next) {
-    CODE maybranch_plop[] = {
-	mov_mem_rebp8};
+unsigned char *
+push_maybranch_plop(unsigned char *code) {
+    unsigned char maybranch_plop[] = {
+	mov_mem_rebx(&PL_op),
+	mov_eax_8ebp};
     PUSHc(maybranch_plop);
-    PUSHrel(&next);
-    return code;
-}
-T_CHARARR maybranch_check[] = {
-    cmp_eax_rebp8,
-    je_0, fourbyte};
-T_CHARARR maybranch_checkw[] = {
-    cmp_eax_rebp8,
-    jew_0, fourbyte};
-CODE *
-push_maybranch_check(CODE *code, int next) {
-    CODE maybranch_check[] = {
-	cmp_eax_rebp8,
-	je_0};
-    if (abs(next) > 128) {
-        CODE maybranch_checkw[] = {
-            cmp_eax_rebp8,
-            jew_0};
-        PUSHc(maybranch_checkw);
-        PUSHrel((CODE*)next);
-    } else {
-        PUSHc(maybranch_check);
-        PUSHbyte(next);
-    }
     return code;
 }
 
@@ -128,7 +108,9 @@ push_maybranch_check(CODE *code, int next) {
 # define CALL	 	x86thr_call
 # define JMP	 	x86thr_call
 # define SAVE_PLOP	x86thr_save_plop
+# define DISPATCH_GETSIG x86thr_dispatch_getsig
 # define DISPATCH       x86thr_dispatch
+# define DISPATCH_POST  x86thr_dispatch_post
 # define EPILOG         x86thr_epilog
 
 /*
